import numpy as np

from .export import ExportConfig
from ..datatypes import StringType, BooleanType, NumberType, FloatType, IntegerType
from ..settings import Sign

class ExportConfigFortran(ExportConfig):

    def parse(self, module:str="ConfigurationModule"):
        lines = []
        lines.append(f"module {module}")
        lines.append("  implicit none")
        lines.append("")
        for name, param in self.data.items():
            name = self._rename(name)
            value = str(param.value)
            if isinstance(param, StringType):
                value = f"\"{value}\""
                dtype = f"character(len={len(value):d})"
            elif isinstance(param, BooleanType):
                value = ".true." if param.value else ".false."
                dtype = "logical"
            elif isinstance(param, IntegerType):
                if param.precision==16: dtype = "integer(kind=2)"
                elif param.precision==32: dtype = "integer"
                elif param.precision==64: dtype = "integer(kind=8)"
            elif isinstance(param, FloatType):
                if param.precision==32:    dtype = "real"
                elif param.precision==64:  dtype = "real(kind=8)"
                elif param.precision==128:  dtype = "real(kind=16)"
            lines.append(f"  {dtype}, parameter :: {name} = {value};")
        lines.append("")
        lines.append(f"end module {module}")
        self.text = Sign.NEWLINE.join(lines)
        return self.text


import numpy as np

from .export import ExportConfig
from ..datatypes import StringType, BooleanType, NumberType, FloatType, IntegerType
from ..settings import Sign

class ExportConfigC(ExportConfig):
    
    def parse(self, guard:str="CONFIG_H"):
        lines = []
        lines.append(f"#ifndef {guard}")
        lines.append(f"#define {guard}")
        lines.append("")
        for name, param in self.data.items():
            name = self._rename(name)
            if param.value is None:
                continue
            value = str(param.value)
            if isinstance(param, StringType):
                value = f"\"{value}\""
            elif isinstance(param, BooleanType):
                if param.value:
                    value = ''
                else:
                    continue
            lines.append(f"#define {name} {value}")
        lines.append("")
        lines.append(f"#endif /* {guard} */")
        self.text = Sign.NEWLINE.join(lines)
        return self.text
    


import numpy as np

from .export import ExportConfig
from ..datatypes import StringType, BooleanType, NumberType, FloatType, IntegerType
from ..settings import Sign

class ExportConfigRust(ExportConfig):
    
    def parse(self):
        lines = []
        for name, param in self.data.items():
            name = self._rename(name)
            value = str(param.value)
            if isinstance(param, StringType):
                value = f"\"{value}\""
                dtype = "&str"
            elif isinstance(param, BooleanType):
                value = "true" if param.value else "false"
                dtype = "bool"
            elif isinstance(param, IntegerType):
                if param.unsigned:
                    if param.precision==8:    dtype = "u8"
                    elif param.precision==16: dtype = "u16"
                    elif param.precision==32: dtype = "u32"
                    elif param.precision==64: dtype = "u64"
                else:
                    if param.precision==8:    dtype = "i8"
                    elif param.precision==16: dtype = "i16"
                    elif param.precision==32: dtype = "i32"
                    elif param.precision==64: dtype = "i64"
            elif isinstance(param, FloatType):
                if param.precision==32:    dtype = "f32"
                elif param.precision==64:  dtype = "f64"
                elif param.precision==128: dtype = "f128"
            lines.append(f"pub const {name}: {dtype} = {value};")
        self.text = Sign.NEWLINE.join(lines)
        return self.text



import numpy as np
import os
import sys
sys.path.insert(0, os.environ['DIR_SOURCE'])

from scinumtools import RowCollector
from scinumtools.units import Dimensions
from scinumtools.units.settings import *
from scinumtools.units.unit_types import *
from scinumtools.dip import DIP
from scinumtools.dip.docs import ExportPDF, ExportHTML

path_docs_static = os.environ['DIR_DOCS']+'/source/_static/tables'

def build_export_pdf():
    
    # Generate a PDF documentation from a DIP file
    dir_pdf = os.environ['DIR_DOCS']+"/source/_static/pdf"
    file_definitions = f"{dir_pdf}/definitions.dip"
    file_cells = f"{dir_pdf}/cells.dip"
    file_pdf = f"{dir_pdf}/documentation.pdf"
    # Create directory if missing
    if not os.path.isdir(dir_pdf):
        os.mkdir(dir_pdf)
    # Create a DIP environment
    with DIP(name="PDF") as p:
        p.add_unit("velocity", 13, 'cm/s')
        p.from_string("""
        $unit length = 1 cm
        $unit mass = 2 g
        
        cfl_factor float = 0.7  # Courant–Friedrichs–Lewy condition
        max_vare float = 0.2    # maximum energy change of electrons
        max_vari float = 0.2    # maximum energy change of ions
        """)
        p.add_source("cells", file_cells)
        p.from_file(file_definitions)
        docs = p.parse_docs()    # Export parameters as a PDF
    with ExportPDF(docs) as exp:
        exp.build(
            file_pdf, 
            "Example DIP documentation", 
            """In this document we want to demonstrate basic capabilities of a DIP documentation.<br/><br/>
            The documentation is structured into 3 main sections. The first section summarizes all parameters
            in a DIP code, as well as their corresponding node definitions, declarations, modifications and corresponding properties.
            Following section summarizes all references of injected values and lists imported nodes.
            The final section gives an overview of custom units and code sources.<br/><br/>
            Parameters, nodes, sections and many other items in this documentation are cross-linked between
            each other. All hyperlinks are denoted with a blue text.
        """)
    print(file_pdf)

def build_export_html():
    
    # Generate a PDF documentation from a DIP file
    dir_html = os.environ['DIR_DOCS']+"/source/_static/htmldocs"
    dir_html_build = f"{dir_html}/build"
    file_definitions = f"{dir_html}/definitions.dip"
    file_cells = f"{dir_html}/cells.dip"
    file_html = f"{dir_html}/documentation.pdf"
    # Create directory if missing
    if not os.path.isdir(dir_html_build):
        os.mkdir(dir_html_build)
    if not os.path.isdir(dir_html):
        os.mkdir(dir_html)
    # Create a DIP environment
    with DIP(name="PDF") as p:
        p.add_unit("velocity", 13, 'cm/s')
        p.from_string("""
        $unit length = 1 cm
        $unit mass = 2 g
        
        cfl_factor float = 0.7  # Courant-Friedrichs-Lewy condition
        max_vare float = 0.2    # maximum energy change of electrons
        max_vari float = 0.2    # maximum energy change of ions
        """)
        p.add_source("cells", file_cells)
        p.from_file(file_definitions)
        docs = p.parse_docs()    # Export parameters as a PDF
    with ExportHTML(docs) as exp:
        exp.build(
            dir_html_build, 
            "Example DIP documentation", 
            """In this document we want to demonstrate basic capabilities of a DIP documentation.<br/><br/>
            The documentation is structured into 3 main sections. The first section summarizes all parameters
            in a DIP code, as well as their corresponding node definitions, declarations, modifications and corresponding properties.
            Following section summarizes all references of injected values and lists imported nodes.
            The final section gives an overview of custom units and code sources.<br/><br/>
            Parameters, nodes, sections and many other items in this documentation are cross-linked between
            each other. All hyperlinks are denoted with a blue text.
        """)
    print(dir_html_build)

def build_prefixes():
    
    # Create table of unit prefixes
    rc = RowCollector(['Symbol','Name','Magnitude'])
    for symbol, prefix in UNIT_PREFIXES.items():
        exponent = int(np.log10(prefix.magnitude))
        rc.append([symbol, prefix.name, ":math:`10^{"+f"{exponent:d}"+"}`"])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/prefixes.csv"
    df.to_csv(file_path, index=False)
    print(file_path)

def build_base_units():

    # create table of base units
    rc = RowCollector(['Symbol','Name','Prefixes'])
    for symbol, unit in UNIT_STANDARD.items():
        if unit.definition is not None:
            continue
        if unit.prefixes is True:
            prefixes = 'all'
        elif isinstance(unit.prefixes, list):
            prefixes = ", ".join([f"{p}{symbol}" for p in unit.prefixes])
        else:
            prefixes = ''
        rc.append([symbol, unit.name, prefixes])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/unit_base.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_standard_units():    

    # create table of standard units
    rc = RowCollector(['Symbol','Name','Definition','Prefixes'])
    for symbol, unit in UNIT_STANDARD.items():
        if not isinstance(unit.definition, str):
            continue
        if symbol in ['K','degR','AR','PR']:
            continue
        if symbol.startswith('['):
            continue
        if unit.prefixes is True:
            prefixes = 'all'
        elif isinstance(unit.prefixes, list):
            prefixes = ", ".join([f"{p}{symbol}" for p in unit.prefixes])
        else:
            prefixes = ''
        if len(rc) and unit.name in rc.Name:
            idx = rc.Name.index(unit.name)
            rc.Symbol[idx] += ", "+symbol
            rc.Prefixes[idx] += ", "+prefixes if rc.Prefixes[idx] else ""
        else:
            rc.append([symbol, unit.name, unit.definition, prefixes])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/unit_standard.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_constants():
    
    # create table of constants
    rc = RowCollector(['Symbol','Name','Definition'])
    for symbol, unit in UNIT_STANDARD.items():
        if not symbol.startswith('['):
            continue
        if len(rc) and unit.name in rc.Name:
            idx = rc.Name.index(unit.name)
            rc.Symbol[idx] += ", "+symbol
        else:
            rc.append([symbol, unit.name, unit.definition])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/constants.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_logarithmic_units():
    
    # create table of logarithmic units
    rc = RowCollector(['Symbol','Name','Prefixes'])
    for symbol, unit in UNIT_STANDARD.items():
        if symbol not in ['AR','PR'] and unit.definition != LogarithmicUnitType:
            continue
        if unit.prefixes is True:
            prefixes = 'all'
        elif isinstance(unit.prefixes, list):
            prefixes = ", ".join([f"{p}{symbol}" for p in unit.prefixes])
        else:
            prefixes = ''
        if len(rc) and unit.name in rc.Name:
            idx = rc.Name.index(unit.name)
            rc.Symbol[idx] += ", "+symbol
            rc.Prefixes[idx] += ", "+prefixes if rc.Prefixes[idx] else ""
        else:
            rc.append([symbol, unit.name, prefixes])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/unit_logarithmic.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_temperature_units():
    
    # create table of temperature units
    rc = RowCollector(['Symbol','Name','Prefixes'])
    for symbol, unit in UNIT_STANDARD.items():
        if symbol not in ['K','degR'] and unit.definition != TemperatureUnitType:
            continue
        if unit.prefixes is True:
            prefixes = 'all'
        elif isinstance(unit.prefixes, list):
            prefixes = ", ".join([f"{p}{symbol}" for p in unit.prefixes])
        else:
            prefixes = ''
        rc.append([symbol, unit.name, prefixes])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/unit_temperature.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_si_quantities():

    # create table of physical quantities SI
    rc = RowCollector(['Symbol','Name','Definition','Dimensions'])
    for q in range(len(QUANTITY_LIST)):
        name = QUANTITY_LIST.name[q]
        symbol = QUANTITY_LIST.symbol[q]
        definition = QUANTITY_LIST.SI[q]
        magnitude, dimensions = QUANTITY_UNITS[f"#S{symbol}"]
        rc.append([f"#S{symbol}", f"SI.{name}", definition, dimensions])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/quantities_si.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_cgs_quantities():
    
    # create table of physical quantities CGS
    rc = RowCollector(['Symbol','Name','Definition'])
    for q in range(len(QUANTITY_LIST)):
        symbol = QUANTITY_LIST.symbol[q]
        if f"#C{symbol}" not in QUANTITY_UNITS:
            continue
        name = QUANTITY_LIST.name[q]
        definition = QUANTITY_LIST.CGS[q]
        rc.append([f"#C{symbol}", f"CGS.{name}", definition])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/quantities_cgs.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    
def build_au_quantities():
    
    # create table of physical quantities AU
    rc = RowCollector(['Symbol','Name','Definition'])
    for q in range(len(QUANTITY_LIST)):
        symbol = QUANTITY_LIST.symbol[q]
        if f"#A{symbol}" not in QUANTITY_UNITS:
            continue
        name = QUANTITY_LIST.name[q]
        definition = QUANTITY_LIST.AU[q]
        rc.append([f"#A{symbol}", f"AU.{name}", definition])
    df = rc.to_dataframe()
    file_path = f"{path_docs_static}/quantities_au.csv"
    df.to_csv(file_path, index=False)
    print(file_path)
    



import numpy as np
import pytest
from math import isclose
import os
import sys
sys.path.insert(0, 'src')

from scinumtools.units import Quantity, Unit
from scinumtools.materials import *

def test_preprocessing():
    
    with MaterialSolver() as ms:
        
        # implicit multiplication and addition
        assert ms.preprocess('C2B4')  == "C * 2 + B * 4"
        assert ms.preprocess('C2 B4') == "C * 2 + B * 4"
        # isotopes and ionizations
        assert ms.preprocess('C{13+2}B{11}H{-}2') == "C{13+2} + B{11} + H{-} * 2"
        # parentheses
        assert ms.preprocess('(CB2)2')                         == "(C + B * 2) * 2"
        assert ms.preprocess('((CB2)2Al)3')                    == "((C + B * 2) * 2 + Al) * 3"
        assert ms.preprocess('C{13+2}(B{11}Li2)4 H{-}2 O{+3}') == "C{13+2} + (B{11} + Li * 2) * 4 + H{-} * 2 + O{+3}"
    
        # preprocessed expressions are constant
        assert ms.preprocess("(C + B * 2) * 2")             == "(C + B * 2) * 2"
        assert ms.preprocess("C{13+2} + (B{11} + Li * 2)4") == "C{13+2} + (B{11} + Li * 2) * 4"

def test_solver():
    
    with MaterialSolver() as ms:
        
        # single atom parsing
        assert str(ms.solve('C'))       == "MaterialPart(p=6 n=6 e=6 m=12.000)"
        assert str(ms.solve('C{13}'))   == "MaterialPart(p=6 n=7 e=6 m=13.003)"
        assert str(ms.solve('C{-2}'))   == "MaterialPart(p=6 n=6 e=4 m=11.999)"
        assert str(ms.solve('C{+}'))    == "MaterialPart(p=6 n=6 e=7 m=12.001)"
        assert str(ms.solve('C{10-}'))  == "MaterialPart(p=6 n=4 e=5 m=10.016)"
        assert str(ms.solve('C{13-2}')) == "MaterialPart(p=6 n=7 e=4 m=13.002)"

        # nucleons
        assert str(ms.solve('[e]'))      == "MaterialPart(p=0 n=0 e=1 m=0.001)"
        assert str(ms.solve('[n]'))      == "MaterialPart(p=0 n=1 e=0 m=1.009)"
        assert str(ms.solve('[n]2'))     == "MaterialPart(p=0 n=2 e=0 m=2.017)"
        assert str(ms.solve('[p]'))      == "MaterialPart(p=1 n=0 e=0 m=1.007)"
        assert str(ms.solve('[p]2'))     == "MaterialPart(p=2 n=0 e=0 m=2.015)"
        assert str(ms.solve('[p]'))      == str(ms.solve('H{1-1}'))
        assert str(ms.solve('[p]B{11}')) == "MaterialPart(p=6 n=6 e=5 m=12.017)"
        
        # hydrogen isotopes
        assert str(ms.solve('H'))       == str(ms.solve('H{1}'))
        assert str(ms.solve('H{+}'))    == str(ms.solve('H{1+}'))
        assert str(ms.solve('D'))       == str(ms.solve('H{2}'))
        assert str(ms.solve('D{-2}'))   == str(ms.solve('H{2-2}'))
        assert str(ms.solve('T'))       == str(ms.solve('H{3}'))
    
        # addition       
        assert str(ms.solve('H{1-1}'))           == "MaterialPart(p=1 n=0 e=0 m=1.007)"
        assert str(ms.solve('B{11}'))            == "MaterialPart(p=5 n=6 e=5 m=11.009)"
        assert str(ms.solve('H{1-1} + B{11}'))   == "MaterialPart(p=6 n=6 e=5 m=12.017)"
    
        # parentheses
        assert str(ms.solve('(H2O)'))   == "MaterialPart(p=10 n=8 e=10 m=18.011)"
        assert str(ms.solve('H(CN)'))   == "MaterialPart(p=14 n=13 e=14 m=27.011)"

        # multiplication
        assert str(ms.solve('H{1-1}2'))             == "MaterialPart(p=2 n=0 e=0 m=2.015)"
        assert str(ms.solve('(H{1-1} + B{11})2'))   == "MaterialPart(p=12 n=12 e=10 m=24.033)"
        
def test_material():
    
    assert str(Material('C')) == "Material(p=6 n=6 e=6 m=12.000)"
    
    # binding energy
    m1 = Quantity(Material('[p][n][e]').mass, 'Da')
    m2 = Quantity(Material('H{2}').mass, 'Da')
    mb = (Quantity(2224.52,'keV')/Unit('[c]2')).to('Da')
    assert str(m1) != str(m2)
    assert m1 == m2 + mb
    #assert Material('He{2}').binding_energy()  == Quantity(2224.581351, 'keV')
    #assert Material('C').binding_energy()/13 == Quantity(7.68, 'MeV')


from bs4 import BeautifulSoup

from ..documentation import Documentation
from .section_parameters import ParametersSection
from .section_references import ReferencesSection
from .section_settings import SettingsSection

class ExportHTML:
        
    docs: Documentation
    
    def __enter__(self):
        return self
    
    def __exit__(self, type, value, traceback):
        pass
    
    def __init__(self, docs: Documentation, **kwargs):
        self.docs = docs
        
    def build(self, dir_html: str, title: str, intro=None):
        
        html = BeautifulSoup("<html><head></head><body></body></html>", features="html5lib")
        
        style = html.new_tag("link", rel="stylesheet", href="https://cdn.jsdelivr.net/npm/bootstrap@4.3.1/dist/css/bootstrap.min.css", integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T", crossorigin="anonymous")
        html.head.append(style)
        
        meta = html.new_tag("meta", charset="UTF-8")
        html.head.append(meta)
        
        container = BeautifulSoup(f"<div class='container'><div class='row'></div></div>", features="html5lib")
        menu = html.new_tag("div", **{'class':'col-xs'})
        content = html.new_tag("div", **{'class':"col"})
        
        section = html.new_tag("h1")
        section.string = title
        content.append(section)
        
        section = html.new_tag("h3")
        section.string = "Table of contents"
        menu.append(section)
        
        sections = {
            "Parameters": {
                'filename':'parameters.html',
                'sections': {
                    "types":"Node types",
                    "parameters":"Parameter list",
                    "nodes":"Parameter nodes"
                }
            },
            "References": {
                'filename':'references.html',
                'sections': {
                    "injections":"Injections",
                    "imports":"Imports"
                }
            },
            "Settings":   {
                'filename':'settings.html',
                'sections': {
                    "units":"Units",
                    "sources":"Sources"
                }
            },
        }
        if intro:
            row = BeautifulSoup(f"<div><a href='./index.html'>Introduction</a></div>", features="html5lib")
            menu.append(row)
        for i2, (name2, settings2) in enumerate(sections.items()):
            row = BeautifulSoup(f"<div><a href='./{settings2['filename']}#SECTION_{name2}'>{name2}</a></div>", features="html5lib")
            menu.append(row)
            for i3, (target,name3) in enumerate(settings2['sections'].items()):
                row = BeautifulSoup(f"<div class='row pl-4'><a href='./{settings2['filename']}#SECTION_{name3}'>{name3}</a></div>", features="html5lib")
                menu.append(row)
        
        if intro:
            section = html.new_tag("h2")
            section.string = "Introduction"
            content.append(section)
            paragraph = html.new_tag("p")
            paragraph.append(BeautifulSoup(intro, features="html5lib"))
            content.append(paragraph)
            
        container.div.div.append(menu)
        container.div.div.append(content)
        container.div.append(BeautifulSoup("<div class='p-3'> </div>", 'html.parser'))
        html.body.append(container)
        
        with open(f"{dir_html}/index.html", "w") as file:
            file.write(str(html.prettify()))
            
        with ParametersSection(self.docs, dir_html, menu) as sect:
            sect.build()
            
        with ReferencesSection(self.docs, dir_html, menu) as sect:
            sect.build()
            
        with SettingsSection(self.docs, dir_html, menu) as sect:
            sect.build()


import sys, os
import pytest
import numpy as np
sys.path.insert(0, 'src')

from scinumtools.dip import DIP

def check_node(node, name, keyword, value):
    assert node.name == name
    assert node.keyword == keyword
    assert node.value.value == value

def test_definition_before_case():
    
    with DIP(docs=True) as p:
        p.from_string("""
        a int = 3        # definition
        @case false
          a float = 4.0  # modification
        @case true
          a str = "4.0"  # modification
        @else
          a bool = true  # modification
        a = 4            # modification
        """)
        env = p.parse_sphinx()
        assert len(env.nodes) == 1
        check_node(env.nodes[0], 'a', 'int', 3)
    
def test_branch_definition():

    # full definition in branch
    with DIP(docs=True) as p:
        p.from_string("""
        @case false
          a float = 4.0  # definition
        @case true
          a str = "4.0"  # definition
        @else
          a bool = true  # definition
        a = 4            # modification
        """)
        env = p.parse_sphinx()
        assert len(env.nodes) == 3
        check_node(env.nodes[0], '@1.a', 'float', 4.0)
        check_node(env.nodes[1], '@2.a', 'str',   "4.0")
        check_node(env.nodes[2], '@3.a', 'bool',  True)

def test_incomplete_case_definition():

    # definition in else case is missing
    with DIP(docs=True) as p:
        p.from_string("""
        @case true
          a float = 4.0  # definition
        a float = 3      # definition, or modification
        """)
        env = p.parse_sphinx()
        assert len(env.nodes) == 2
        check_node(env.nodes[0], '@1.a', 'float', 4.0)
        check_node(env.nodes[1], 'a',    'float', 3)

    # all but one branch has definition
    with DIP(docs=True) as p:
        p.from_string("""
        @case true
          b float = 4.0  
        @else
          a float = 5.0  # definition
        a float = 3      # modification
        """)
        env = p.parse_sphinx()
        assert len(env.nodes) == 3
        check_node(env.nodes[0], '@1.b', 'float', 4.0)
        check_node(env.nodes[1], '@2.a', 'float', 5.0)
        check_node(env.nodes[2], 'a',    'float', 3.0)

def test_nested_incomplete():

    # incomplete nested definition
    with DIP(docs=True) as p:
        p.from_string("""
        @case true
          @case true
            b float = 4.0  
          @else
            a float = 8.0  # definition
        @else
          a float = 5.0  # definition
        a float = 3      # modification
        """)
        env = p.parse_sphinx()
        assert len(env.nodes) == 4
        check_node(env.nodes[0], '@1.@2.b', 'float', 4.0)
        check_node(env.nodes[1], '@1.@3.a', 'float', 8.0)
        check_node(env.nodes[2], '@4.a',    'float', 5.0)
        check_node(env.nodes[3], 'a',       'float', 3.0)



import numpy as np

from .export import ExportConfig
from ..datatypes import StringType, BooleanType, NumberType, FloatType, IntegerType
from ..settings import Sign, Format
from ..environment import Environment

class ExportConfigBash(ExportConfig):
    
    def __init__(self, env: Environment, **kwargs):
        if 'dtype' not in kwargs:
            kwargs['dtype'] = Format.VALUE
        super().__init__(env, **kwargs)
        
    def parse_value(self, value):
        if isinstance(value, str):
            value = f"\"{value}\""
        elif isinstance(value, bool):
            value = "0" if value else "-1"   # in bash 0 is true and usually 1, -1 for error
        return value
    
    def parse(self, export=True):
        lines = []
        for name, value in self.data.items():
            name = self._rename(name)
            
            if isinstance(value, (list, np.ndarray)):
                value = "\" \"".join([str(self.parse_value(val)) for val in value])
                value = f"(\"{value}\")"
            else:
                value = self.parse_value(value)

            if export:
                lines.append(f"export {name}={value}")
            else:
                lines.append(f"{name}={value}")
        self.text = Sign.NEWLINE.join(lines)
        return self.text


import sys, os
import pytest
import numpy as np
sys.path.insert(0, 'src')

from scinumtools.dip import DIP
from scinumtools.dip.docs import ExportPDF

@pytest.fixture
def file_pdf():
    dir_tmp = "tmp" #docs/source/_static/pdf"
    if not os.path.isdir(dir_tmp):
        os.mkdir(dir_tmp)
    file_pdf  = f"{dir_tmp}/documentation.pdf"
    if os.path.isfile(file_pdf):
        os.remove(file_pdf)
    return file_pdf

@pytest.fixture
def file_definitions():
    return "../../docs/source/_static/pdf/definitions.dip"
    
@pytest.fixture
def file_cells():
    return "../../docs/source/_static/pdf/cells.dip"

def test_export_pdf(file_pdf, file_definitions, file_cells):
    with DIP() as p:
        p.add_unit("velocity", 13, 'cm/s')
        p.from_string("""
        $unit length = 1 cm
        $unit mass = 2 g
        
        cfl_factor float = 0.7  # Courant–Friedrichs–Lewy condition
        max_vare float = 0.2    # maximum energy change of electrons
        max_vari float = 0.2    # maximum energy change of ions
        """)
        p.add_source("cells", file_cells)
        p.from_file(file_definitions)
        docs = p.parse_docs()
    with ExportPDF(docs) as exp:
        title = "Example documentation"
        pageinfo = "DIP Documentation"
        exp.build(file_pdf, title, pageinfo)




import numpy as np

from .export import ExportConfig
from ..datatypes import StringType, BooleanType, NumberType, FloatType, IntegerType
from ..settings import Sign

class ExportConfigCPP(ExportConfig):
    
    def parse(self, guard="CONFIG_H"):
        lines = []
        lines.append(f"#ifndef {guard}")
        lines.append(f"#define {guard}")
        lines.append("")
        for name, param in self.data.items():
            name = self._rename(name)
            value = str(param.value)
            if isinstance(param, StringType):
                value = f"\"{value}\""
                dtype = "const char*"
            elif isinstance(param, BooleanType):
                value = "true" if param.value else "false"
                dtype = "bool"
            elif isinstance(param, IntegerType):
                if param.unsigned:
                    if param.precision==16:   dtype = "unsigned short int"
                    elif param.precision==32: dtype = "unsigned int"
                    elif param.precision==64: dtype = "unsigned long long int"
                else:
                    if param.precision==8:    dtype = "signed char"
                    elif param.precision==16: dtype = "short int"
                    elif param.precision==32: dtype = "int"
                    elif param.precision==64: dtype = "long long int"
            elif isinstance(param, FloatType):
                if param.precision==32: dtype = "float"
                elif param.precision==64: dtype = "double"
                elif param.precision in [80,96,128]: dtype = "long double"
            lines.append(f"constexpr {dtype} {name} = {value};")
        lines.append("")
        lines.append(f"#endif /* {guard} */")
        self.text = Sign.NEWLINE.join(lines)
        return self.text